//==============================================
//  EDID Emulator
//  arch_avr.h
//
//  2023/05/15 Ver 0.0
//  2024/05/25 Ver 0.1 Arduino AVR Borads 1.8.6 by Arduino
//
//  Copyright (C)2023-2024 FREE WING, Y.Sakamoto
//
//  http://www.neko.ne.jp/~freewing/
//==============================================

#ifndef __ARCH_AVR_INC__
#define __ARCH_AVR_INC__

extern "C" {
  #include "utility/twi.h"
}
#if defined(TWI_BUFFER_LENGTH) && TWI_BUFFER_LENGTH < 128
  #if !defined(TWI_BUFFER_LENGTH_TX) || TWI_BUFFER_LENGTH_TX < 128
    // https://github.com/arduino/ArduinoCore-avr/blob/63092126a406402022f943ac048fa195ed7e944b/libraries/Wire/src/utility/twi.h#L34
    #error "Error: TWI_BUFFER_LENGTH_TX patch is required, Edit /libraries/Wire/src/utility/twi.h and twi.cpp"
  #endif
#endif

#define IS_AVR

#define USE_SOFTWIRE

// #define NOT_USE_SERIAL

// #define NOT_USE_INT

// #define MAX_EDID_TABLE 8

  // Target Board
  // ATmega32U4 Pro Micro +5V tested OK
  // https://learn.sparkfun.com/tutorials/pro-micro--fio-v3-hookup-guide
  // (as Arduino Micro)
  // https://store.arduino.cc/products/arduino-micro

  // ATmega168P Pro Mini BTE20-14 +5V tested OK
  // (as Arduino Pro Mini clone, Arduino Mini 05 clone)
  // https://docs.arduino.cc/retired/boards/arduino-pro-mini
  // https://docs.arduino.cc/retired/boards/arduino-mini-05-without-header

  // ATmega328P Arduino Pro Mini +5V can compile, but not tested
  // https://docs.arduino.cc/retired/boards/arduino-pro-mini

  // ATmega168PB Pro Mini BTE22-02 +5V can compile, but not tested
  // (as Arduino Nano factor)
  // https://store.arduino.cc/products/arduino-nano

// #if defined(BUFFER_LENGTH) && BUFFER_LENGTH < 128
//   // https://github.com/arduino/ArduinoCore-avr/blob/master/libraries/Wire/src/Wire.h#L29
//   #error "Error: BUFFER_LENGTH define is not 128, Edit /libraries/Wire/src/Wire.h"
// #endif

#include <AceWire.h>

#if defined(__AVR_ATmega32U4__)
  #define USER_LED 16 /** IO16, PB2, MOSI */
  #define LED_OFF HIGH
  #define LED_ON  LOW

  #define USER_BUTTON 10 /** IO10, PB6 */
  #define BUTTON_OFF HIGH
  #define BUTTON_ON  LOW

  #define HPD_COMPUTER 8 /** IO8, PB4 */
  #define HPD_COMPUTER_OUTPUT OUTPUT
  #define HPD_DISPLAY 9 /** IO9, PB5 */
  #define HPD_DISPLAY_INPUT INPUT /** with External Pull-Down */
  #define HPD_ENABLE HIGH
  #define HPD_DISABLE LOW

  // I2C Master for Read Display EDID info.
  #define I2C0_SDA 4 /** PD4 */
  #define I2C0_SCL 5 /** PC6 */

  // I2C Master for OLED
  #define OLED_SDA 6 /** PD7 */
  #define OLED_SCL 7 /** PE6 */

  // Wire = I2C Native
  // SDA D2, PD1
  // SCL D3, PD0

  #ifndef NOT_USE_INT
    #define AVR_PCICR  0b00000001 /** Enable Port B pin Change Interrupt */
    // #define AVR_PCMSK0 (digitalPinToBitMask(USER_BUTTON) | digitalPinToBitMask(HPD_DISPLAY))
    #define AVR_PCMSK0 0b01100000 /** bit 6 = PB6(D10), bit 5 = PB5(D9) */
  #endif

#endif

#if defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168__)
  #define USER_LED 13 /** D13, PB5 */
  #define LED_OFF LOW
  #define LED_ON  HIGH

  #define USER_BUTTON 10 /** D10, PB2 */
  #define BUTTON_OFF HIGH
  #define BUTTON_ON  LOW

  #define HPD_COMPUTER 8 /** D8, PB0 */
  #define HPD_COMPUTER_OUTPUT OUTPUT
  #define HPD_DISPLAY 9 /** D9, PB1 */
  #define HPD_DISPLAY_INPUT INPUT /** with External Pull-Down */
  #define HPD_ENABLE HIGH
  #define HPD_DISABLE LOW

  // I2C Master for Read Display EDID info.
  #define I2C0_SDA 4 /** PD4 */
  #define I2C0_SCL 5 /** PD5 */

  // I2C Master for OLED
  #define OLED_SDA 6 /** PD6 */
  #define OLED_SCL 7 /** PD7 */

  // Wire = I2C Native
  // SDA A4, PC4
  // SCL A5, PC5

  #ifndef NOT_USE_INT
    #define AVR_PCICR  0b00000001 /** Enable Port B pin Change Interrupt */
    // #define AVR_PCMSK0 (digitalPinToBitMask(USER_BUTTON) | digitalPinToBitMask(HPD_DISPLAY))
    #define AVR_PCMSK0 0b00000110 /** bit 2 = PB2(D10), bit 1 = PB1(D9) */
  #endif

#endif

#if !defined(USER_BUTTON)
  #error "Error: Not Support this AVR Board"
#endif

// #if defined(__AVR_ATmega168__)
//   #define NOT_USE_SERIAL
// #endif

  // Slave
  #define WireSlave  Wire

  // Master
  const uint8_t DELAY_MICROS = (1000000UL / I2C_FREQ); // 10us = 100kHz
  using WireInterface = ace_wire::SimpleWireInterface;

  WireInterface wireInterfaceMaster(I2C0_SDA, I2C0_SCL, DELAY_MICROS);
  #define WireMaster wireInterfaceMaster

// #if !defined(__AVR_ATmega168__)
  WireInterface wireInterfaceOled(OLED_SDA, OLED_SCL, DELAY_MICROS);
  #define WireOled wireInterfaceOled
// #endif

#if defined(WireOled)
// WireOled
void WireOled_begin() {
  // WireOled = AceWire
  WireOled.begin();
}

void WireOled_end() {
  WireOled.end();
}
#endif

// WireMaster
void WireMaster_begin() {
  // WireMaster = AceWire
  WireMaster.begin();
}

void WireMaster_end() {
  WireMaster.end();
}

// WireSlave
void WireSlave_begin() {
  // Wire = I2C Native

  // Slave
  WireSlave.begin(EDID_I2C_ADDRESS);

  // I2C with internal pullups
  // https://forum.arduino.cc/t/i2c-with-internal-pullups/355691
  // pinMode(SDA, INPUT);
  // pinMode(SCL, INPUT);

  WireSlave.onReceive(onReceive);
  WireSlave.onRequest(onRequest);
}

#endif
