//==============================================
//  EDID Emulator
//  arch_esp32.h
//
//  2023/05/15 Ver 0.0
//  2024/05/25 Ver 0.1 esp32 2.0.17 by Espressif
//
//  Copyright (C)2023-2024 FREE WING, Y.Sakamoto
//
//  http://www.neko.ne.jp/~freewing/
//==============================================

#ifndef __ARCH_ESP32_INC__
#define __ARCH_ESP32_INC__

  // Target Board
  // ESP32-S2 WeMos S2 mini tested OK
  // https://www.wemos.cc/en/latest/s2/s2_mini.html

  // ESP32-C3 ESP32-C3-DevKitM-1 tested OK
  // https://docs.espressif.com/projects/esp-idf/en/latest/esp32c3/hw-reference/esp32c3/user-guide-devkitm-1.html

  // ESP32-S3 can compile, but not tested
  // ESP32-D0 can compile, but not tested

#if defined(I2C_BUFFER_LENGTH) && I2C_BUFFER_LENGTH < 128
  // https://github.com/espressif/arduino-esp32/blob/d164df89de350aa945ccb0cb4261f56306bfdb3c/libraries/Wire/src/Wire.h#L48
  #error "Error: I2C_BUFFER_LENGTH define is not 128, Edit /libraries/Wire/src/Wire.h"
#endif

#ifdef CONFIG_IDF_TARGET_ESP32C3
  #define USE_SOFTWIRE
#endif

#if CONFIG_IDF_TARGET_ESP32S2
  #define USER_LED 15
  #define LED_OFF LOW
  #define LED_ON  HIGH

  #define USER_BUTTON 0
  #define BUTTON_OFF HIGH
  #define BUTTON_ON  LOW

  #define HPD_COMPUTER 10
  #define HPD_COMPUTER_OUTPUT OUTPUT
  #define HPD_DISPLAY  8
  #define HPD_DISPLAY_INPUT INPUT
  #define HPD_ENABLE HIGH
  #define HPD_DISABLE LOW

  // I2CEXT0
  // I2C Master for Read Display EDID info.
  #define I2C0_SDA 3
  #define I2C0_SCL 5

  // I2CEXT0
  // I2C Master for OLED
  #define OLED_SDA 7
  #define OLED_SCL 9

  // I2CEXT1
  // Wire1 = I2C2
  #define I2C1_SDA 4
  #define I2C1_SCL 6
#endif

#ifdef CONFIG_IDF_TARGET_ESP32C3
  // #define USER_LED 8
  // #define LED_OFF LOW
  // #define LED_ON  HIGH

  #define USER_BUTTON 9
  #define BUTTON_OFF HIGH
  #define BUTTON_ON  LOW

  #define HPD_COMPUTER 19
  #define HPD_COMPUTER_OUTPUT OUTPUT
  #define HPD_DISPLAY  18
  #define HPD_DISPLAY_INPUT INPUT
  #define HPD_ENABLE HIGH
  #define HPD_DISABLE LOW

  // AceWire
  // I2C Master for Read Display EDID info.
  #define I2C0_SDA 7
  #define I2C0_SCL 6

  // AceWire
  // I2C Master for OLED
  #define OLED_SDA 0
  #define OLED_SCL 1

  // I2CEXT0
  // I2C Slave
  #define I2C1_SDA 5
  #define I2C1_SCL 4
#endif

#if !defined(USER_BUTTON)
  #error "Error: Not Support this ESP32 Board"
#endif

  // Master
  #ifdef USE_SOFTWIRE
    #include <AceWire.h>

    const uint8_t DELAY_MICROS = (1000000UL / I2C_FREQ); // 10us = 100kHz
    using WireInterface = ace_wire::SimpleWireInterface;

    WireInterface wireInterfaceMaster(I2C0_SDA, I2C0_SCL, DELAY_MICROS);
    #define WireMaster wireInterfaceMaster

    WireInterface wireInterfaceOled(OLED_SDA, OLED_SCL, DELAY_MICROS);
    #define WireOled wireInterfaceOled

    // Slave
    #define WireSlave Wire

  #else
    #define WireMaster Wire
    #define WireOled Wire

    // Slave
    #define WireSlave Wire1
  #endif

// WireOled
void WireOled_begin() {
  // ESP32
#ifdef USE_SOFTWIRE
  // WireOled = AceWire
  WireOled.begin();
#else
  // WireOled = I2CEXT0
  int sda = OLED_SDA;
  int scl = OLED_SCL;
  WireOled.setPins(sda, scl);
  WireOled.begin(-1, -1, I2C_FREQ); // OK
#endif
}

void WireOled_end() {
  WireOled.end();
}

// WireMaster
void WireMaster_begin() {
#ifdef USE_SOFTWIRE
  // WireMaster = AceWire
  WireMaster.begin();
#else
  // WireMaster = I2CEXT0
  WireMaster.setPins(I2C0_SDA, I2C0_SCL);
  WireMaster.begin(-1, -1, I2C_FREQ); // OK
#endif
}

void WireMaster_end() {
  WireMaster.end();
}

// WireSlave
void WireSlave_begin() {
  // WireSlave = I2CEXT1
  WireSlave.setPins(I2C1_SDA, I2C1_SCL);

  // ESP32 Before calling `begin()` two callback functions
  // https://espressif-docs.readthedocs-hosted.com/projects/arduino-esp32/en/latest/api/i2c.html#i2c-slave-mode
  WireSlave.onReceive(onReceive);
  WireSlave.onRequest(onRequest);

  // Slave
  // WireSlave.begin(EDID_I2C_ADDRESS);
  WireSlave.begin(EDID_I2C_ADDRESS, -1, -1, I2C_FREQ); // OK

#if defined(I2C_FREQ)
  // ESP32 Slave no setClock()
  // https://github.com/espressif/arduino-esp32/blob/master/libraries/Wire/src/Wire.cpp#L385
#endif

#if CONFIG_IDF_TARGET_ESP32
  // For ESP32 only !
  // https://espressif-docs.readthedocs-hosted.com/projects/arduino-esp32/en/latest/api/i2c.html#slavewrite
  byte dmy[128];
  Wire.slaveWrite((uint8_t *)dmy, 128);
#endif
}

#endif
