//==============================================
//  EDID Emulator
//  arch_lgt8.h
//
//  2023/05/15 Ver 0.0
//  2024/05/25 Ver 0.1 LGT8fxBoards 2.0.7 by dbuezas
//
//  Copyright (C)2023 FREE WING, Y.Sakamoto
//
//  http://www.neko.ne.jp/~freewing/
//==============================================

#ifndef __ARCH_LGT_INC__
#define __ARCH_LGT_INC__

extern "C" {
  #include "utility/twi.h"
}
#if defined(TWI_BUFFER_LENGTH) && TWI_BUFFER_LENGTH < 128
  #if !defined(TWI_BUFFER_LENGTH_TX) || TWI_BUFFER_LENGTH_TX < 128
    // https://github.com/dbuezas/lgt8fx/blob/c88788c9b062677cddb25aafafc1775ec61e84a4/lgt8f/libraries/Wire/utility/twi.h#L34
    #error "Error: TWI_BUFFER_LENGTH_TX patch is required, Edit /libraries/Wire/src/utility/twi.h and twi.cpp"
  #endif
#endif

#define IS_AVR

#define USE_SOFTWIRE

// #define NOT_USE_SERIAL

// #define NOT_USE_INT

// #define MAX_EDID_TABLE 8

  // Target Board
  // LGT8F328P LQFP32 MiniEVB Nano3 BTE21-15A +5V Ext. 16MHz tested OK
  // (as Arduino Nano clone)
  // https://store.arduino.cc/products/arduino-nano

  // LGT8F328P LQFP32 MiniEVB BTE17-14 can compile, but not tested
  // (as Arduino Pro Mini clone, Arduino Mini 05 clone)
  // https://docs.arduino.cc/retired/boards/arduino-pro-mini
  // https://docs.arduino.cc/retired/boards/arduino-mini-05-without-header

  // LGT8F168P can compile, but not tested

#include <AceWire.h>

#if defined(__LGT8FX8P__ )
  #define USER_LED D13 /** PB5 */
  #define LED_OFF LOW
  #define LED_ON  HIGH

  #define USER_BUTTON D10 /** PB2 */
  #define BUTTON_OFF HIGH
  #define BUTTON_ON  LOW

  #define HPD_COMPUTER D8 /** PB0 */
  #define HPD_COMPUTER_OUTPUT OUTPUT
  #define HPD_DISPLAY D9 /** PB1 */
  #define HPD_DISPLAY_INPUT INPUT /** with External Pull-Down */
  #define HPD_ENABLE HIGH
  #define HPD_DISABLE LOW

  // I2C Master for Read Display EDID info.
  #define I2C0_SDA D4 /** PD4 */
  #define I2C0_SCL D5 /** PD5 */

  // I2C Master for OLED
  #define OLED_SDA D6 /** PD6 */
  #define OLED_SCL D7 /** PD7 */

  // I2C Master for OLED
  // #define OLED_SDA A2 /** PC2 */
  // #define OLED_SCL A3 /** PC3 */

  // Wire = I2C Native
  // SDA A4, PC4
  // SCL A5, PC5

  #ifndef NOT_USE_INT
    #define AVR_PCICR  0b00000001 /** Enable Port B pin Change Interrupt */
    // #define AVR_PCMSK0 (digitalPinToBitMask(USER_BUTTON) | digitalPinToBitMask(HPD_DISPLAY))
    #define AVR_PCMSK0 0b00000110 /** bit 2 = PB2(D10), bit 1 = PB1(D9) */
  #endif

#endif

#if !defined(USER_BUTTON)
  #error "Error: Not Support this LGT8 Board"
#endif

  // Slave
  #define WireSlave  Wire

  // Master
  const uint8_t DELAY_MICROS = (1000000UL / I2C_FREQ); // 10us = 100kHz
  using WireInterface = ace_wire::SimpleWireInterface;

  WireInterface wireInterfaceMaster(I2C0_SDA, I2C0_SCL, DELAY_MICROS);
  #define WireMaster wireInterfaceMaster

  WireInterface wireInterfaceOled(OLED_SDA, OLED_SCL, DELAY_MICROS);
  #define WireOled wireInterfaceOled

#if defined(WireOled)
// WireOled
void WireOled_begin() {
  // WireOled = AceWire
  WireOled.begin();
}

void WireOled_end() {
  WireOled.end();
}
#endif

// WireMaster
void WireMaster_begin() {
  // WireMaster = AceWire
  WireMaster.begin();
}

void WireMaster_end() {
  WireMaster.end();
}

// WireSlave
void WireSlave_begin() {
  // Wire = I2C Native

  // Slave
  WireSlave.begin(EDID_I2C_ADDRESS);

  // I2C with internal pullups
  // https://forum.arduino.cc/t/i2c-with-internal-pullups/355691
  // pinMode(SDA, INPUT);
  // pinMode(SCL, INPUT);

  WireSlave.onReceive(onReceive);
  WireSlave.onRequest(onRequest);
}

#endif
