//==============================================
//  EDID Emulator
//  arch_rp2040.h
//
//  2023/05/15 Ver 0.0
//  2024/05/25 Ver 0.1 Raspberry Pi Pico/RP2040 3.8.1 by Earle F. Philhower, III
//
//  Copyright (C)2023-2024 FREE WING, Y.Sakamoto
//
//  http://www.neko.ne.jp/~freewing/
//==============================================

#ifndef __ARCH_RP2040_INC__
#define __ARCH_RP2040_INC__

  // Target Board
  // RP2040 YD-RP2040 tested OK
  // http://152.32.187.208:8080/yd-data/YD-RP2040/

  #define USER_LED D25
  #define LED_OFF LOW
  #define LED_ON  HIGH

  #define USER_BUTTON D24
  #define BUTTON_OFF HIGH
  #define BUTTON_ON  LOW

  #define HPD_COMPUTER D9
  #define HPD_COMPUTER_OUTPUT OUTPUT
  #define HPD_DISPLAY D8
  #define HPD_DISPLAY_INPUT INPUT
  #define HPD_ENABLE HIGH
  #define HPD_DISABLE LOW

  // I2C Master for Read Display EDID info.
  #define I2C0_SDA D0
  #define I2C0_SCL D1

  // I2C Master for OLED
  #define OLED_SDA D20
  #define OLED_SCL D21

 // Wire1 = I2C2
  #define I2C1_SDA D2
  #define I2C1_SCL D3

  // Master
  #define WireMaster Wire

  // Slave
  #define WireSlave  Wire1

  // Master
  #define WireOled Wire

// WireOled
void WireOled_begin() {
  // Select IO pins to use. Call before ::begin()
  WireOled.setSDA(OLED_SDA);
  WireOled.setSCL(OLED_SCL);
  WireOled.begin();
}

void WireOled_end() {
  WireOled.end();
}

// WireMaster
void WireMaster_begin() {
  // Wire = I2C0
  // Select IO pins to use.  Call before ::begin()
  WireMaster.setSDA(I2C0_SDA);
  WireMaster.setSCL(I2C0_SCL);

#if defined(I2C_FREQ)
  WireMaster.setClock(I2C_FREQ);
#endif

  WireMaster.begin();
}

void WireMaster_end() {
  WireMaster.end();
}

// WireSlave
void WireSlave_begin() {
  // Wire1 = I2C1
  // Select IO pins to use.  Call before ::begin()
  WireSlave.setSDA(I2C1_SDA);
  WireSlave.setSCL(I2C1_SCL);

#if defined(I2C_FREQ)
  WireSlave.setClock(I2C_FREQ);
#endif

  // Slave
  WireSlave.begin(EDID_I2C_ADDRESS);

  // RP2040 (Before or After any)
  WireSlave.onReceive(onReceive);
  WireSlave.onRequest(onRequest);
}

#endif
