//==============================================
//  EDID Emulator
//  arch_stm32.h
//
//  2023/05/15 Ver 0.0
//  2024/05/25 Ver 0.1 STM32 MCU based boards 2.7.1 by STMicroelectronics
//
//  Copyright (C)2023-2024 FREE WING, Y.Sakamoto
//
//  http://www.neko.ne.jp/~freewing/
//==============================================

#ifndef __ARCH_STM32_INC__
#define __ARCH_STM32_INC__

#if defined(BUFFER_LENGTH) && BUFFER_LENGTH < 128
  // https://github.com/stm32duino/Arduino_Core_STM32/blob/0e9d1084df6d4998f0f2a664d9b8b33bc78bf329/libraries/Wire/src/Wire.h#L36
  #error "Error: BUFFER_LENGTH define is not 128, Edit /libraries/Wire/src/Wire.h"
#endif
#if defined(I2C_TXRX_BUFFER_SIZE) && I2C_TXRX_BUFFER_SIZE < 128
  // https://github.com/stm32duino/Arduino_Core_STM32/blob/0e9d1084df6d4998f0f2a664d9b8b33bc78bf329/libraries/Wire/src/utility/twi.h#L65
  #error "Error: I2C_TXRX_BUFFER_SIZE define is not 128, Edit /libraries/Wire/src/utility/twi.h"
#endif

  // Target Board
  // STM32F103C8T6 Blue Pill tested OK
  // https://stm32-base.org/boards/STM32F103C8T6-Blue-Pill

  // STM32F103C8T6 Black Pill can compile, but not tested
  // https://stm32-base.org/boards/STM32F103C8T6-Black-Pill

  // STM32F401CEU6/ STM32F401CCU6 WeAct Black Pill V3.0 tested OK
  // https://stm32-base.org/boards/STM32F401CEU6-WeAct-Black-Pill-V3.0.html

  // STM32F401RCT6 Unknown BlackPill can compile, but not tested

#if defined(STM32F1xx)
  #define USER_LED PC13 /** Blue Pill */
  // #define USER_LED PB12 /** Black Pill */

  #define USER_BUTTON PA0

  // STM32 is Fixed GPIO pins
  // Wire = I2C1
  // Wire default SDA = PB7, SCL = PB6
  //#define I2C1_REMAP
  #ifndef I2C1_REMAP
    // I2C Master for Read Display EDID info.
    #define I2C0_SDA PB7 /** GPIO PB7 */
    #define I2C0_SCL PB6 /** GPIO PB6 */

    // I2C Master for OLED
    #define OLED_SDA PB9 /** GPIO PB9 */
    #define OLED_SCL PB8 /** GPIO PB8 */
  #else
    #define I2C0_SDA PB9 /** REMAP GPIO PB9 */
    #define I2C0_SCL PB8 /** REMAP GPIO PB8 */

    // I2C Master for OLED
    #define OLED_SDA PB7 /** REMAP GPIO PB7 */
    #define OLED_SCL PB6 /** REMAP GPIO PB6 */
  #endif

  // Wire1 = I2C2
  #define I2C1_SDA PB11 /** GPIO PB11 */
  #define I2C1_SCL PB10 /** GPIO PB10 */
#endif

#if defined(STM32F4xx)
  #define USER_LED PC13

  #define USER_BUTTON PA0

  // STM32 is Fixed GPIO pins
  // Wire = I2C1
  // Wire default SDA = PB7, SCL = PB6
  // I2C Master for Read Display EDID info.
  #define I2C0_SDA PB7 /** GPIO PB7 */
  #define I2C0_SCL PB6 /** GPIO PB6 */

  // Wire2 = I2C3
  // I2C Master for OLED
  #define OLED_SDA PB4 /** GPIO PB4 */
  #define OLED_SCL PA8 /** GPIO PA8 */
  #define MULTIPLE_I2C_PORTS_WIRE2

  // Wire1 = I2C2
  #define I2C1_SDA PB3  /** GPIO PB3 */
  #define I2C1_SCL PB10 /** GPIO PB10 */
#endif

#if !defined(USER_BUTTON)
  #error "Error: Not Support this STM32 Board"
#endif

  #define LED_OFF HIGH
  #define LED_ON  LOW

  #define BUTTON_OFF HIGH
  #define BUTTON_ON  LOW

  #define HPD_COMPUTER PA9
  #define HPD_COMPUTER_OUTPUT OUTPUT

  // Don't use Internal pull-down
  // FT = 5 V tolerant. To sustain a voltage higher than VDD + 0.3 V the internal pull-up/pull-down resistors must be disabled.
  #define HPD_DISPLAY PA10
  #define HPD_DISPLAY_INPUT INPUT /** with External Pull-Down */

  #define HPD_ENABLE HIGH
  #define HPD_DISABLE LOW

  // Slave
  TwoWire Wire1; // I2C2
  #define WireSlave  Wire1

  // Enable SoftWire
  // #define USE_SOFTWIRE_MASTER

  #ifdef USE_SOFTWIRE_MASTER
    #include <AceWire.h>

    const uint8_t DELAY_MICROS = (1000000UL / I2C_FREQ); // 10us = 100kHz
    using WireInterface = ace_wire::SimpleWireInterface;
  
    WireInterface wireInterfaceOled(I2C0_SDA, I2C0_SCL, DELAY_MICROS);
    #define WireMaster wireInterfaceOled    
  #else

    // Master
    #define WireMaster Wire /** with External Pull-Up */
  #endif

#if defined(OLED_SDA)
  #if defined(I2C3_BASE)
    TwoWire Wire2; // I2C3
    #define WireOled Wire2
  #else
    #define WireOled Wire
  #endif
#endif

#if defined(WireOled)
// WireOled
void WireOled_begin() {
  // STM32F103C8T6 Wire
  // STM32F40x Wire2
  WireOled.setSDA(OLED_SDA);
  WireOled.setSCL(OLED_SCL);
  WireOled.begin();
#if defined(I2C_FREQ)
  WireOled.setClock(I2C_FREQ);
#endif
}

void WireOled_end() {
  WireOled.end();
}
#endif

// WireMaster
void WireMaster_begin() {
#ifdef USE_SOFTWIRE_MASTER
  // WireMaster = AceWire
  WireMaster.begin();
#else
  // Wire = I2C1
  // Select IO pins to use. Call before ::begin()
  WireMaster.setSDA(I2C0_SDA);
  WireMaster.setSCL(I2C0_SCL);
  WireMaster.begin();
#if defined(I2C_FREQ)
  WireMaster.setClock(I2C_FREQ);
#endif
#endif
}

void WireMaster_end() {
  WireMaster.end();
}

// WireSlave
void WireSlave_begin() {
  // Wire1 = I2C2
  // Select IO pins to use. Call before ::begin()
  WireSlave.setSDA(I2C1_SDA);
  WireSlave.setSCL(I2C1_SCL);

  // Slave
  WireSlave.begin(EDID_I2C_ADDRESS);

  // STM32 After calling `begin()` two callback functions
  WireSlave.onReceive(onReceive);
  WireSlave.onRequest(onRequest);

#if defined(I2C_FREQ)
  WireSlave.setClock(I2C_FREQ);
#endif
}

#endif
