//==============================================
//  EDID Emulator
//  arch_uno_r4.h
//
//  2023/09/15 Ver 0.0
//  2024/05/25 Ver 0.1 Arduino UNO R4 Borads 1.1.0 by Arduino
//
//  Copyright (C)2023 FREE WING, Y.Sakamoto
//
//  http://www.neko.ne.jp/~freewing/
//==============================================

#ifndef __ARCH_RA4M1_INC__
#define __ARCH_RA4M1_INC__

  // Target Board
  // Arduino UNO R4 Minima clone tested OK
  // https://store-usa.arduino.cc/products/uno-r4-minima

  #define USER_LED LED_BUILTIN /** UNO R4 Minima */

  #define USER_BUTTON PIN_D2

  // UNO R4 is Fixed GPIO pins
  // Wire = I2C1
  // Wire default SDA = PB7, SCL = PB6
    // I2C Master for Read Display EDID info.
    #define I2C0_SDA PIN_D11 // P109 SDA9
    #define I2C0_SCL PIN_D12 // P110 SCL9 

    // I2C Master for OLED
    #define OLED_SDA PIN_D8 // P304
    #define OLED_SCL PIN_D9 // P303

  // Wire1 = I2C2
  #define I2C1_SDA SDA // P101/ SDA1/ A4
  #define I2C1_SCL SCL // P100/ SCL1/ A5

  #define LED_OFF LOW
  #define LED_ON  HIGH

  #define BUTTON_OFF HIGH
  #define BUTTON_ON  LOW

  #define HPD_COMPUTER PIN_D3 // P104
  #define HPD_COMPUTER_OUTPUT OUTPUT

  // RA4M1 No Internal pull-down
  #define HPD_DISPLAY PIN_D10 // P112
  #define HPD_DISPLAY_INPUT INPUT /** with External Pull-Down */

  #define HPD_ENABLE HIGH
  #define HPD_DISABLE LOW

  // Slave
  TwoWire Wire1(I2C1_SCL, I2C1_SDA);
  #define WireSlave  Wire1

  // Enable SoftWire
  #define USE_SOFTWIRE_MASTER

  #ifdef USE_SOFTWIRE_MASTER
    // Blacklist platforms using https://github.com/arduino/ArduinoCore-api due to
    // incompatibilities.
    // #error Platforms using ArduinoCore-API not supported
    #undef ARDUINO_API_VERSION
    #include <AceWire.h>

    const uint8_t DELAY_MICROS = (1000000UL / I2C_FREQ); // 10us = 100kHz
    using WireInterface = ace_wire::SimpleWireInterface;

    WireInterface wireInterfaceMaster(I2C0_SDA, I2C0_SCL, DELAY_MICROS);
    #define WireMaster wireInterfaceMaster

    #if defined(OLED_SDA)
      #define USE_SOFTWIRE

      WireInterface wireInterfaceOled(OLED_SDA, OLED_SCL, DELAY_MICROS);
      #define WireOled wireInterfaceOled
    #endif
  #endif

#if defined(WireOled)
// WireOled = AceWire
void WireOled_begin() {
  WireOled.begin();
}

void WireOled_end() {
  WireOled.end();
}
#endif

// WireMaster
void WireMaster_begin() {
  // WireMaster = AceWire
  WireMaster.begin();
}

void WireMaster_end() {
  WireMaster.end();
}

// WireSlave
void WireSlave_begin() {
  // Wire1 = I2C2

  // Slave
  WireSlave.begin(EDID_I2C_ADDRESS);

  WireSlave.onReceive(onReceive);
  WireSlave.onRequest(onRequest);

#if defined(I2C_FREQ)
  WireSlave.setClock(I2C_FREQ);
#endif
}

#endif
