//==============================================
//  EDID Emulator
//  oled.h
//
//  2023/05/15 Ver 0.0
//  2024/05/25 Ver 0.1
//
//  Copyright (C)2023-2024 FREE WING, Y.Sakamoto
//
//  http://www.neko.ne.jp/~freewing/
//==============================================

#ifndef __OLED_INC__
#define __OLED_INC__

bool isOledWorking = false;

// OLED SSD1306
// https://github.com/ThingPulse/esp8266-oled-ssd1306
// #include <SSD1306Wire.h>
// #include "Font_7x5.V3 (23)_slim.h"
// Program Size too BIG !

#if defined(WireOled)
  // https://github.com/greiman/SSD1306Ascii
  // Small footprint OLED library
  #define INCLUDE_SCROLLING 1 /** It MUST 1*/
  #include "SSD1306Ascii.h"

  #ifdef USE_SOFTWIRE
    #define MULTIPLE_I2C_PORTS_WITH_TEMPLATE 1 /** for Assign SoftWire */
    #include "SSD1306AsciiWireT.h"
    // use AceWire
    SSD1306AsciiWireT<WireInterface*> oled = SSD1306AsciiWireT<WireInterface*>(&WireOled);
  #else
    #ifdef MULTIPLE_I2C_PORTS_WIRE2
      #define MULTIPLE_I2C_PORTS true
      #include "SSD1306AsciiWire.h"
      // use Wire2 I2C3
      SSD1306AsciiWire oled = SSD1306AsciiWire(Wire2);
    #else
      #include "SSD1306AsciiWire.h"
      // use Wire
      SSD1306AsciiWire oled;
    #endif
  #endif
#endif

// I2C OLED SSD1306 64x32 0.49 inch
#define OLED_DEVICE MicroOLED64x32

#if defined(WireOled)
void Oled_begin() {
  WireOled_begin();

  // OLED I2C SSD1306 64x32 0.49
  oled.begin(&OLED_DEVICE, OLED_I2C_ADDRESS);
#ifdef OLED_BRIGHTNESS
  oled.setContrast(OLED_BRIGHTNESS);
#endif
  oled.setFont(System5x7);

  isOledWorking = true;
}

void Oled_clear() {
  oled.clear();
}

void Oled_attr(int16_t xMove, int16_t yMove, bool invert, bool mag2x) {
  if (xMove >= 0) oled.setCursor(xMove * 6, yMove);
  oled.setInvertMode(invert);
  mag2x? oled.set2X() : oled.set1X();
}

void Oled_print(int16_t xMove, int16_t yMove, const char ch, bool invert = false, bool mag2x = false) {
  Oled_attr(xMove, yMove, invert, mag2x);
  oled.write(ch);
}

void Oled_print(int16_t xMove, int16_t yMove, const char* strUser, bool invert = false, bool mag2x = false) {
  Oled_attr(xMove, yMove, invert, mag2x);
  oled.print(strUser);
}

void Oled_write_P(const char* strUser) {
  PGM_P p = strUser;
  char ch;
  while((ch = pgm_read_byte(p++)) != 0x00) {
    oled.write(ch);
  }
}

// for F() macro
void Oled_print(int16_t xMove, int16_t yMove, const __FlashStringHelper* strUser, bool invert = false, bool mag2x = false) {
  Oled_attr(xMove, yMove, invert, mag2x);
  Oled_write_P((const char*)strUser);
}

#if defined(IS_AVR)
// for PROGMEM
void Oled_print_P(int16_t xMove, int16_t yMove, const char* const* strUser, bool invert = false, bool mag2x = false) {
  Oled_attr(xMove, yMove, invert, mag2x);
  Oled_write_P((const char*)pgm_read_word(strUser));
}
#endif

void Oled_end() {
  WireOled_end();

  isOledWorking = false;
}

#else
  // Dummy function
  #define Oled_begin()
  #define Oled_clear()
  #define Oled_end()

  void Oled_print(int16_t xMove, int16_t yMove, const char ch, bool invert = false, bool mag2x = false) {
  }

  void Oled_print(int16_t xMove, int16_t yMove, const char* strUser, bool invert = false, bool mag2x = false) {
  }

  void Oled_print(int16_t xMove, int16_t yMove, const __FlashStringHelper* strUser, bool invert = false, bool mag2x = false) {
  }

  #if defined(IS_AVR)
  void Oled_print_P(int16_t xMove, int16_t yMove, const char* const* strUser, bool invert = false, bool mag2x = false) {
  }
  #endif
#endif

#endif
